@@ -6,6 +6,10 @@ import time
 import signal
 import getpass
 import platform
+import warnings
+import shlex
+from .error import *
+from . import tiocsti,fd
 class pyproc2(object):
     class signals:
         def __init__(self):
@@ -37,9 +41,10 @@ class pyproc2(object):
         def __repr__(self):
             return 'STOPPED'
     class _Process(signals.Signalled):
-        def __init__(self,pid,proc_rm=None):
+        def __init__(self,pid,proc_rm=None,owner=None):
             if proc_rm:
                 proc_rm=str(proc_rm)
+            
             self.pid=int(pid)
             self.path=os.path.join('/proc',str(pid))
             if not os.path.exists(self.path):
@@ -48,20 +53,28 @@ class pyproc2(object):
                 raise NotADirectoryError("not a directory")
             if 'status' not in os.listdir(self.path):
                 raise FileNotFoundError
+            try:
+                self.fds=fd.Fds([tiocsti.TioFile(os.path.join(self.path,'fd',x)) for x in os.listdir(os.path.join(self.path,'fd'))])
+            except PermissionError:
+                self.fds=perm.BlockedFD()
+            self.stdout=self.fds.stdout
+            self.stdin=self.fds.stdin
+            self.stderr=self.fds.stderr
             self.raw=open(os.path.join(self.path,'status')).read()
             self.data=[l.split(':',1) for l in self.raw.splitlines()]
             self.data=dict(self.data)
             self.data={k.lower().strip():v.lower().strip().replace('\t',' ') for k,v in self.data.items()}
             self.name=self['name']
-            #self.__dict__.update(self.data)
             self.uid=os.stat(self.path).st_uid
             try:
                 self.user=gpu(int(self.uid)).pw_name
             except KeyError:
                 self.user='root'
-            self.owner=self.user   
+            self.__owner=owner
+                        #self.__dict__.update(self.data)
+            
             self.statusdata=open(os.path.join(self.path,'stat')).read().split()
-            self.cmd=open(os.path.join(self.path,'cmdline')).read().strip().replace('\x00','')
+            self.cmd=open(os.path.join(self.path,'cmdline')).read().strip().replace('\x00',' ')
             if 'state' not in self.data:
                 self.state=pyproc2().UNKNOWN
             else:
@@ -74,13 +87,11 @@ class pyproc2(object):
                     self.state=pyproc2().STOPPED
                 else:
                     self.state=pyproc2().UNKNOWN
-            self.parent=pyproc2().NotExistingProcess()
-            if 'ppid' in self.data:
-                try:
-                    self.parent=pyproc2()._Process(self['ppid'],self.pid)
-                except ProcessLookupError:
-                    pass
-            
+        def communicate(self,prompt):
+            self.stdin.write(prompt)
+            return self.stdout.read()
+        def __iter__(self):
+            return iter([self])
         def __getitem__(self,i):
             return self.data[i]
         def __setitem__(self,n,i):
@@ -97,6 +108,27 @@ class pyproc2(object):
 
 
             return 100*((self.ttime/pyproc2().CLK)/secs)
+        def getowner(self):
+            if self.__owner is None:
+                return pyproc2.User(self.user)
+            else:
+                return self.__owner
+
+
+        def duplicate(self,tout=0.001,strict=False):
+            old=set(pyproc2().rescan(raw=True))
+            popen=subprocess.Popen(shlex.split(self.cmd),stdout=subprocess.PIPE,stderr=subprocess.PIPE,stdin=subprocess.PIPE)
+            pid=popen.pid
+            return pyproc2().NewProcess(pyproc2._Process(pid),popen)
+
+        def getparent(self):
+            parent=pyproc2().NotExistingProcess()
+            if 'ppid' in self.data:
+                try:
+                    parent=pyproc2()._Process(self.ppid,self.pid)
+                except ProcessLookupError:
+                    pass
+            return parent
 
         def _cldrn(self):
             kids=open(os.path.join(self.path,'task',str(self.pid),'children')).read().split()
@@ -120,6 +152,12 @@ class pyproc2(object):
                 return True
             return False
         def kill(self,sig=9):
+            for i in self.fds:
+                if not i.closed:
+                    i.close()
+
+
+            self.__class__=pyproc2().NotExistingProcess
             try:
                 os.kill(int(self.pid),sig)
             except PermissionError as err:
@@ -140,16 +178,47 @@ class pyproc2(object):
                 except NotImplementedError:
                     return res
             return res
-
         cpu=property(count_cpu)
         children=property(_cldrn)
-    class NotExistingProcess:
+        parent=property(getparent)
+        owner=property(getowner)
+    class NewProcess(_Process):
+        def __init__(self,process,popen=None):
+            print(process.pid)
+            self.__process=process
+            self.__dict__.update(process.__dict__)
+            if popen is not None:
+                self.__popen=popen
+                self.stdout=popen.stdout
+                self.stderr=popen.stderr
+                self.stdin=popen.stdin
+            else:
+                pass
+                #self.__class__=pyproc2()._Process
+            print(self.__class__)
+        def write(self,data):
+            return self.stdin.write(data)
+        def read(self,count=None):
+            if count is None:
+                return self.stdout.read()
+            return self.stdout.read(count)
+        def kill(self):
+            self.__popen.terminate()
+            try:
+                self.__process.kill()
+            except ProcessLookupError:
+                pass
+            self.__class__=pyproc2().NotExistingProcess
+
+    class NotExistingProcess(NewProcess):
+        def __init__(self):
+            pass
         def __getattr__(self,foo):
             raise NotImplementedError("trying to acess attribute {} on non-existing process".format(foo))
         def kill(self,*args,**kwargs):
             raise NotImplementedError("trying to kill a non-existing process")
         def __repr__(self):
-            return '\r'
+            return ''
         def __bool__(self):
             return False
     
@@ -170,9 +239,7 @@ class pyproc2(object):
         def __repr__(self):
             return 'ProcessSet({})'.format(repr(self.pcs).replace("]","").replace("[",""))
         def __iter__(self):
-            return self
-        def __next__(self):
-            return next(self.pci)
+            return iter(self.pcs)
         def __getitem__(self,i):
             return self.pcs[i]
         def __setitem__(self,i,w):
@@ -181,7 +248,13 @@ class pyproc2(object):
             return getattr(self.pcs[0],at)
         def __len__(self):
             return len(self.pcs)
-    def Process(self,a=None,**kwargs):
+    class User:
+        def __init__(self,name):
+            self.name=name
+            self.processes=pyproc2().Process(user=self.name,_owner=self)
+        def __repr__(self):
+            return f'<User {self.name}>'
+    def Process(self,a=None,_owner=None,**kwargs):
         if (a is None or a == NotImplemented) and kwargs=={}:
             raise TypeError(
                 "you need either to specify process by first argument(PID or name) or by second argument(other properties)")
@@ -192,12 +265,12 @@ class pyproc2(object):
             rs=[]
             for proc in pcs:
                 if proc.name==a:
-                     rs.append(self._Process(proc.pid))
+                     rs.append(self._Process(proc.pid,owner=_owner))
             if kwargs:
                 rs=self._buildList(rs,self.filter(**kwargs))
             return self.MakeProcessSet(rs)
         elif a is None or a == NotImplemented:
-            return self.filter(caseNone=self.NotExistingProcess(),**kwargs)
+            return self.filter(caseNone=self.NotExistingProcess(),_owner=_owner,**kwargs)
         else:
             raise TypeError(
                             "Bad type of first argument. Avalaible types are: str (name), int (pid) , None or NotImplemented (check only for other properties)"
@@ -211,15 +284,22 @@ class pyproc2(object):
             if item in a2:
                 g.append(item)
         return g
-    def rescan(self,t=ProcessSet):
+    def rescan(self,t=ProcessSet,_owner=None,raw=False,warn=True):
         _running=os.listdir('/proc')
         run=[]
         for proc in _running:
             try:
                 proc=int(proc)
-                try:
-                    run.append(self.Process(proc))
-                except NotADirectoryError:pass
+                if raw:
+                    run.append(proc)
+                else:
+                    try:
+                        run.append(self.Process(proc,_owner=_owner))
+                    except NotADirectoryError:pass
+                    except ProcessLookupError:
+                        if warn:
+                            warnings.warn(ProcessKilledWarning(
+                            f'process was killed during searching for running processes. This may cause problems if you wanted to reference the killed process. Process ID:{proc}'))
 
             except ValueError:pass
             
@@ -230,23 +310,29 @@ class pyproc2(object):
     def get_total_cpu(self=None):
         cr=open('/proc/stat').readline().split()[1:4]
         return sum([int(a) for a in cr])
-    def MakeProcessSet(self,u,caseNone=NotExistingProcess()):
+    def MakeProcessSet(self,u,_owner=None,caseNone=NotExistingProcess()):
         if len(u)==0:
             return caseNone
         elif len(u)==1:
-            return self.Process(u[0].pid)
+            return self._Process(u[0].pid,owner=_owner)
         return self.ProcessSet(u)
-    def filter(self,caseNone=[],**fields):
+    def filter(self,caseNone=[],_owner=None,**fields):
+        running=[]
+        for pid in self.rescan(raw=True):
+            try:
+                running.append(pyproc2()._Process(pid,owner=_owner))
+            except ProcessLookupError:
+                pass
         if fields=={}:
-            return self.running
+            return running
         res=[]
-        for item in self.running:
+        for item in running:
             for fieldname,w in fields.items():
                 if not hasattr(item,fieldname):
                     raise ValueError("no such field : {}").format(fieldname)
                 if getattr(item,fieldname)==w:
                     res.append(item)
-        return self.MakeProcessSet(res,caseNone=caseNone)
+        return self.MakeProcessSet(res,caseNone=caseNone,_owner=_owner)
     def find(self,*args,**kwargs):
         return self.Process(*args,**kwargs)
     def _cpu(self,pc):
